import requests
import datetime

from bs4 import BeautifulSoup
from .base import APIError


BASE_URL = "https://libcal.library.upenn.edu"
API_URL = "https://api2.libcal.com"

LOCATION_BLACKLIST = set([3620, 2636, 2611, 3217, 2637, 2634])
ROOM_BLACKLIST = set([7176, 16970, 16998, 17625])


class StudySpaces(object):
    """Used for interacting with the library libcal api.

    Usage::

      >>> from penn import StudySpaces
      >>> s = StudySpaces('LIBCAL_CLIENT_ID', 'LIBCAL_CLIENT_SECRET')
    """

    def __init__(self, client_id, client_secret):
        self.token = None
        self.expiration = None
        self.client_id = client_id
        self.client_secret = client_secret

    def _obtain_token(self):
        """Obtain an auth token from client id and client secret."""

        # don't renew token if hasn't expired yet
        if self.expiration and self.expiration > datetime.datetime.now():
            return

        resp = requests.post("{}/1.1/oauth/token".format(API_URL), data={
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "grant_type": "client_credentials"
        }).json()

        if "error" in resp:
            raise APIError("LibCal Auth Failed: {}, {}".format(resp["error"], resp.get("error_description")))

        self.expiration = datetime.datetime.now() + datetime.timedelta(seconds=resp["expires_in"])
        self.token = resp["access_token"]

    def _request(self, *args, **kwargs):
        """Make a signed request to the libcal API."""
        if not self.token:
            self._obtain_token()

        headers = {
            "Authorization": "Bearer {}".format(self.token)
        }

        # add authorization headers
        if "headers" in kwargs:
            kwargs["headers"].update(headers)
        else:
            kwargs["headers"] = headers

        # add api site to url
        args = list(args)
        if not args[1].startswith("http"):
            args[1] = "{}{}".format(API_URL, args[1])

        has_no_token = kwargs.get("no_token")
        if has_no_token:
            del kwargs["no_token"]
        resp = requests.request(*args, **kwargs)
        if resp.status_code == 401 and not has_no_token:
            self._obtain_token()
            kwargs["no_token"] = True
            self._request(*args, **kwargs)
        return resp

    def get_buildings(self):
        """Returns a list of location IDs and names."""
        resp = self._request("GET", "/1.1/space/locations").json()
        out = []
        for x in resp:
            if x["lid"] in LOCATION_BLACKLIST:
                continue
            if x["public"] == 1:
                del x["public"]
                x["service"] = "libcal"
                out.append(x)
        return out

    def get_rooms(self, lid, start=None, end=None):
        """Returns a list of rooms and their availabilities, grouped by category.

        :param lid: The ID of the location to retrieve rooms for.
        :type lid: int
        :param start: The start range for the availabilities to retrieve, in YYYY-MM-DD format.
        :type start: str
        :param end: The end range for the availabilities to retrieve, in YYYY-MM-DD format.
        :type end: str
        """
        range_str = "availability"
        if start:
            start_datetime = datetime.datetime.combine(datetime.datetime.strptime(start, "%Y-%m-%d").date(), datetime.datetime.min.time())
            range_str += "=" + start
            if end and not start == end:
                range_str += "," + end
        else:
            start_datetime = None

        resp = self._request("GET", "/1.1/space/categories/{}".format(lid)).json()
        if "error" in resp:
            raise APIError(resp["error"])
        output = {"id": lid, "categories": []}

        # if there aren't any rooms associated with this location, return
        if len(resp) < 1:
            return output

        if "error" in resp[0]:
            raise APIError(resp[0]["error"])

        if "categories" not in resp[0]:
            return output

        categories = resp[0]["categories"]
        id_to_category = {i["cid"]: i["name"] for i in categories}
        categories = ",".join([str(x["cid"]) for x in categories])
        resp = self._request("GET", "/1.1/space/category/{}".format(categories))
        for category in resp.json():
            cat_out = {"cid": category["cid"], "name": id_to_category[category["cid"]], "rooms": []}

            # ignore equipment categories
            if cat_out["name"].endswith("Equipment"):
                continue

            items = category["items"]
            items = ",".join([str(x) for x in items])
            resp = self._request("GET", "/1.1/space/item/{}?{}".format(items, range_str))
            for room in resp.json():
                if room["id"] in ROOM_BLACKLIST:
                    continue
                # prepend protocol to urls
                if "image" in room and room["image"]:
                    if not room["image"].startswith("http"):
                        room["image"] = "https:" + room["image"]
                # convert html descriptions to text
                if "description" in room:
                    description = room["description"].replace(u'\xa0', u' ')
                    room["description"] = BeautifulSoup(description, "html.parser").text.strip()
                # remove extra fields
                if "formid" in room:
                    del room["formid"]
                # enforce date filter
                # API returns dates outside of the range, fix this manually
                if start_datetime:
                    out_times = []
                    for time in room["availability"]:
                        parsed_start = datetime.datetime.strptime(time["from"][:-6], "%Y-%m-%dT%H:%M:%S")
                        if parsed_start >= start_datetime:
                            out_times.append(time)
                    room["availability"] = out_times
                cat_out["rooms"].append(room)
            if cat_out["rooms"]:
                output["categories"].append(cat_out)
        return output

    def book_room(self, item, start, end, fname, lname, email, nickname, custom={}, test=False):
        """Books a room given the required information.

        :param item:
            The ID of the room to book.
        :type item: int
        :param start:
            The start time range of when to book the room, in the format returned by the LibCal API.
        :type start: str
        :param end:
            The end time range of when to book the room, in the format returned by the LibCal API.
        :type end: str
        :param fname:
            The first name of the user booking the room.
        :type fname: str
        :param lname:
            The last name of the user booking the room.
        :type lname: str
        :param email:
            The email of the user booking the room.
        :type email: str
        :param nickname:
            The name of the reservation.
        :type nickname: str
        :param custom:
            Any other custom fields required to book the room.
        :type custom: dict
        :param test:
            If this is set to true, don't actually book the room. Default is false.
        :type test: bool
        :returns:
            Dictionary containing a success and error field.
        """
        data = {
            "start": start,
            "fname": fname,
            "lname": lname,
            "email": email,
            "nickname": nickname,
            "bookings": [
                {
                    "id": item,
                    "to": end
                }
            ],
            "test": test
        }
        data.update(custom)
        resp = self._request("POST", "/1.1/space/reserve", json=data)
        out = resp.json()
        if "errors" in out and "error" not in out:
            errors = out["errors"]
            if isinstance(errors, list):
                errors = " ".join(errors)
            out["error"] = BeautifulSoup(errors.replace("\n", " "), "html.parser").text.strip()
            del out["errors"]
        if "results" not in out:
            if "error" not in out:
                out["error"] = None
                out["results"] = True
            else:
                out["results"] = False
        return out

    def cancel_room(self, booking_id):
        """Cancel a room given a booking id.

        :param booking_id: A booking id or a list of booking ids (separated by commas) to cancel.
        :type booking_id: str
        """
        resp = self._request("POST", "/1.1/space/cancel/{}".format(booking_id))
        return resp.json()

    def get_reservations(self, email, date):
        """Gets reservations for a given email.

        :param email: the email of the user who's reservations are to be fetched
        :type email: str
        """
        try:
            resp = self._request("GET", "/1.1/space/bookings?email={}&date={}&limit=100".format(email, date))
        except resp.exceptions.HTTPError as error:
            raise APIError("Server Error: {}".format(error))
        return resp.json()

    def get_reservations_for_booking_ids(self, booking_ids):
        """Gets booking information for a given list of booking ids.

        :param booking_ids: a booking id or a list of room ids (comma separated).
        :type booking_ids: string
        """
        try:
            resp = self._request("GET", "/1.1/space/booking/{}".format(booking_ids))
        except resp.exceptions.HTTPError as error:
            raise APIError("Server Error: {}".format(error))
        return resp.json()

    def get_room_info(self, room_ids):
        """Gets room information for a given list of ids.

        :param room_ids: a room id or a list of room ids (comma separated).
        :type room_ids: string
        """
        try:
            resp = self._request("GET", "/1.1/space/item/{}".format(room_ids))
            rooms = resp.json()
            for room in rooms:
                if not room["image"].startswith("http"):
                    room["image"] = "https:" + room["image"]

                if "description" in room:
                    description = room["description"].replace(u'\xa0', u' ')
                    room["description"] = BeautifulSoup(description, "html.parser").text.strip()
        except resp.exceptions.HTTPError as error:
            raise APIError("Server Error: {}".format(error))
        return rooms



from os import path
from nameparser import HumanName
from .base import WrapperBase


BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/"
ENDPOINTS = {
    'SEARCH': BASE_URL + 'directory',
    'DETAILS': BASE_URL + 'directory_person_details',
}


class Directory(WrapperBase):
    """The client for the Directory. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import Directory
      >>> d = Directory('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """
    @staticmethod
    def standardize(res):
        # Standardize name
        name = HumanName(res['list_name'])
        name.capitalize()
        res['list_name'] = str(name)
        if 'detail_name' in res:
            dname = HumanName(res['detail_name'])
            dname.capitalize()
            res['detail_name'] = str(dname)
        # Lowercase email
        if 'list_email' in res:
            res['list_email'] = res['list_email'].lower()
        # Remove `Faculty - ` from affiliation
        res['list_affiliation'] = res['list_affiliation'].replace('Faculty - ', '')
        return res

    def search(self, params, standardize=False):
        """Get a list of person objects for the given search params.

        :param params: Dictionary specifying the query parameters
        :param standardize: Whether to standardize names and other features,
            currently disabled for backwards compatibility. Currently
            standardizes names, lowercases emails, and removes faculty label
            from affiliation.

        >>> people = d.search({'first_name': 'tobias', 'last_name': 'funke'})
        """
        resp = self._request(ENDPOINTS['SEARCH'], params)
        if not standardize:
            return resp
        # Standardization logic
        for res in resp['result_data']:
            res = self.standardize(res)
        return resp

    def detail_search(self, params, standardize=False):
        """Get a detailed list of person objects for the given search params.

        :param params:
            Dictionary specifying the query parameters

        >>> people_detailed = d.detail_search({'first_name': 'tobias', 'last_name': 'funke'})
        """

        response = self._request(ENDPOINTS['SEARCH'], params)
        result_data = []
        for person in response['result_data']:
            try:
                detail = self.person_details(person['person_id'],
                                             standardize=standardize)
            except ValueError:
                pass
            else:
                result_data.append(detail)

        response['result_data'] = result_data
        return response

    def person_details(self, person_id, standardize=False):
        """Get a detailed person object

        :param person_id:
            String corresponding to the person's id.

        >>> instructor = d.person('jhs878sfd03b38b0d463b16320b5e438')
        """
        resp = self._request(path.join(ENDPOINTS['DETAILS'], person_id))
        if standardize:
            resp['result_data'] = [self.standardize(res) for res in resp['result_data']]
        return resp



import os
import csv
import requests
import pkg_resources
from bs4 import BeautifulSoup

LAUNDRY_DOMAIN = os.environ.get("LAUNDRY_DOMAIN", "suds.kite.upenn.edu")
ALL_URL = 'http://{}/?location='.format(LAUNDRY_DOMAIN)
USAGE_BASE_URL = 'https://www.laundryalert.com/cgi-bin/penn6389/LMRoomUsage?CallingPage=LMRoom&Password=penn6389&Halls='


class Laundry(object):
    """The client for Laundry. Used to make requests to the API.

    Usage::

      >>> from penn import Laundry
      >>> l = Laundry()
    """

    def __init__(self):
        self.busy_dict = {
            'LowBusyNightColor': 'Low',
            'LowBusyDayColor': 'Low',
            'MediumLowBusyColor': 'Medium',
            'MediumHighBusyColor': 'High',
            'HighBusyColor': 'Very High',
            'NoDataBusyColor': 'No Data'
        }
        self.days = ['Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday',
                     'Saturday', 'Sunday']
        self.hall_to_link = {}
        self.id_to_hall = {}
        self.id_to_location = {}
        self.hall_id_list = []
        self.create_hall_to_link_mapping()

    def create_hall_to_link_mapping(self):
        """
        :return: Mapping from hall name to associated link in SUDS. Creates inverted index from id to hall.
        """
        laundry_path = pkg_resources.resource_filename("penn", "data/laundry.csv")
        with open(laundry_path, "r") as f:
            reader = csv.reader(f)
            for row in reader:
                hall_id, hall_name, location, uuid = row
                hall_id = int(hall_id)
                self.hall_to_link[hall_name] = ALL_URL + uuid
                self.id_to_hall[hall_id] = hall_name
                self.id_to_location[hall_id] = location
                self.hall_id_list.append({"hall_name": hall_name, "id": hall_id, "location": location})

    @staticmethod
    def update_machine_object(cols, machine_object):
        if cols[2].getText() == "In use" or cols[2].getText() == "Almost done":
            time_remaining = cols[3].getText().split(" ")[0]
            machine_object["running"] += 1
            try:
                machine_object["time_remaining"].append(int(time_remaining))
            except ValueError:
                pass
        elif cols[2].getText() == "Out of order":
            machine_object["out_of_order"] += 1
        elif cols[2].getText() == "Not online":
            machine_object["offline"] += 1
        else:
            machine_object["open"] += 1

        # edge case that handles machine not sending time data
        diff = int(machine_object["running"]) - len(machine_object["time_remaining"])
        while diff > 0:
            machine_object["time_remaining"].append(-1)
            diff = diff - 1

        return machine_object

    def parse_a_hall(self, hall):
        """Return names, hall numbers, and the washers/dryers available for a certain hall.

        :param hall:
            The ID of the hall to retrieve data for.
        :type hall: int
        """
        if hall not in self.hall_to_link:
            return None  # change to to empty json idk
        page = requests.get(self.hall_to_link[hall], timeout=60)
        soup = BeautifulSoup(page.content, 'html.parser')
        soup.prettify()
        washers = {"open": 0, "running": 0, "out_of_order": 0, "offline": 0, "time_remaining": []}
        dryers = {"open": 0, "running": 0, "out_of_order": 0, "offline": 0, "time_remaining": []}

        detailed = []

        rows = soup.find_all('tr')
        for row in rows:
            cols = row.find_all('td')
            if len(cols) > 1:
                machine_type = cols[1].getText()
                if machine_type == "Washer":
                    washers = Laundry.update_machine_object(cols, washers)
                elif machine_type == "Dryer":
                    dryers = Laundry.update_machine_object(cols, dryers)
                if machine_type in ["Washer", "Dryer"]:
                    try:
                        time = int(cols[3].getText().split(" ")[0])
                    except ValueError:
                        time = 0
                    detailed.append({
                        "id": int(cols[0].getText().split(" ")[1][1:]),
                        "type": cols[1].getText().lower(),
                        "status": cols[2].getText(),
                        "time_remaining": time
                    })

        machines = {"washers": washers, "dryers": dryers, "details": detailed}
        return machines

    def all_status(self):
        """Return names, hall numbers, and the washers/dryers available for all
        rooms in the system

        >>> all_laundry = l.all_status()
        """
        laundry_rooms = {}
        for room in self.hall_to_link:
            laundry_rooms[room] = self.parse_a_hall(room)

        return laundry_rooms

    def hall_status(self, hall_id):
        """Return the status of each specific washer/dryer in a particular
        laundry room.

        :param hall_id:
             Integer corresponding to the id of the hall. This id
             is returned as part of the all_status call.

        >>> english_house = l.hall_status("English%20House")
        """
        if hall_id not in self.id_to_hall:
            raise ValueError("No hall with id %s exists." % hall_id)

        hall_name = self.id_to_hall[hall_id]
        location = self.id_to_location[hall_id]
        machines = self.parse_a_hall(hall_name)

        return {
            'machines': machines,
            'hall_name': hall_name,
            'location': location
        }

    def machine_usage(self, hall_no):
        """Returns the average usage of laundry machines every hour
        for a given hall.

        The usages are returned in a dictionary, with the key being
        the day of the week, and the value being an array listing the usages
        per hour.

        :param hall_no:
             integer corresponding to the id number for the hall. Thus number
             is returned as part of the all_status call.

        >>> english_house = l.machine_usage(2)
        """

        try:
            num = int(hall_no)
        except ValueError:
            raise ValueError("Room Number must be integer")
        r = requests.get(USAGE_BASE_URL + str(num), timeout=60)
        parsed = BeautifulSoup(r.text, 'html5lib')
        usage_table = parsed.find_all('table', width='504px')[0]
        rows = usage_table.find_all('tr')
        usages = {}
        for i, row in enumerate(rows):
            day = []
            hours = row.find_all('td')
            for hour in hours:
                day.append(self.busy_dict[str(hour['class'][0])])
            usages[self.days[i]] = day
        return usages



"""A module for consuming the Penn Transit API"""
from .base import WrapperBase

BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/transit/"
ENDPOINTS = {
    'APC': BASE_URL + 'apc',
    'MDT': BASE_URL + 'mdt',
    'TRANSAPC': BASE_URL + 'transapc',
    'STOPINVENTORY': BASE_URL + 'stopinventory',
    'STOPTIMES': BASE_URL + 'stoptimes',
    'PREDICTION': BASE_URL + '511/Prediction',
    'CONFIGURATION': BASE_URL + '511/Configuration'
}


class Transit(WrapperBase):
    """The client for Transit. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import Transit
      >>> trans = Transit('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """

    @staticmethod
    def format_date(date):
        return date.strftime("%m/%d/%Y") + " " + date.strftime("%H:%M:%S")

    def apc(self, start_date, end_date):
        """Return all APC data packets in date range

        :param start_date:
            The starting date for the query.
        :param end_date:
            The end date for the query.
        >>> import datetime
        >>> today = datetime.date.today()
        >>> trans.apc(today - datetime.timedelta(days=1), today))
        """
        params = {
            'start': self.format_date(start_date),
            'end': self.format_date(end_date)
        }
        response = self._request(ENDPOINTS['APC'], params)
        return response

    def mdt(self, start_date, end_date):
        """Return all MDT data packets in date range

        :param start_date:
            The starting date for the query.
        :param end_date:
            The end date for the query.
        >>> import datetime
        >>> today = datetime.date.today()
        >>> trans.mdt(today - datetime.timedelta(days=1), today))
        """

        params = {
            'start': self.format_date(start_date),
            'end': self.format_date(end_date)
        }
        response = self._request(ENDPOINTS['MDT'], params)
        return response

    def transapc(self, start_date, end_date):

        """Return detail of boardings, alightings, by vehicle and stop,
        including the passenger load leaving the stop (this is only for
        vehicles equipped with APC hardware)

        :param start_date:
            The starting date for the query.
        :param end_date:
            The end date for the query.
        >>> import datetime
        >>> today = datetime.date.today()
        >>> trans.transapc(today - datetime.timedelta(days=1), today))

        """
        params = {
            'start': self.format_date(start_date),
            'end': self.format_date(end_date)
        }
        response = self._request(ENDPOINTS['TRANSAPC'], params)
        return response

    def stopinventory(self):
        """Return a list all transit stops.

        >>> stops = trans.stopinventory()
        """
        response = self._request(ENDPOINTS['STOPINVENTORY'])
        return response

    def prediction(self):
        """Return route data and time predictions

        >>> predictions = trans.prediction()
        """
        response = self._request(ENDPOINTS['PREDICTION'])
        return response

    def configuration(self):
        """Return route configuration info

        >>> route_config = trans.configuration()
        """
        response = self._request(ENDPOINTS['CONFIGURATION'])
        return response

    def stoptimes(self, start_date, end_date):
        """Return all stop times in the date range

        :param start_date:
            The starting date for the query.
        :param end_date:
            The end date for the query.
        >>> import datetime
        >>> today = datetime.date.today()
        >>> trans.stoptimes(today - datetime.timedelta(days=1), today)
        """
        params = {
            'start': self.format_date(start_date),
            'end': self.format_date(end_date)
        }
        response = self._request(ENDPOINTS['STOPTIMES'], params)
        return response



import re
import requests
import datetime

from bs4 import BeautifulSoup
from .base import APIError

BASE_URL = "https://apps.wharton.upenn.edu/gsr"


class Wharton(object):
    """Used for interacting with the Wharton GSR site.

    Usage::

      >>> from penn import Wharton
      >>> s = Wharton()
    """

    def get_reservations(self, sessionid):
        """Returns a list of location IDs and names."""
        url = "{}{}".format(BASE_URL, "/reservations/")
        cookies = dict(sessionid=sessionid)

        try:
            resp = requests.get(url, cookies=cookies)
        except resp.exceptions.HTTPError as error:
            raise APIError("Server Error: {}".format(error))

        html = resp.content.decode("utf8")

        if "https://weblogin.pennkey.upenn.edu" in html:
            raise APIError("Wharton Auth Failed. Session ID is not valid.")

        soup = BeautifulSoup(html, "html5lib")
        reservations = []
        media = soup.find_all("div", {'class': "Media-body"})
        for res in media:
            times = res.find_all("span", {'class': "list-view-item__end-time"})
            reservation = {
                "date": res.find("span", {'class': "list-view-item__start-time u-display-block"}).get_text(),
                "startTime": times[0].get_text(),
                "endTime": times[1].get_text(),
                "location": res.find("span", {'class': "list-view-item-building"}).get_text(),
                "booking_id": int(res.find("a")['href'].split("delete/")[1][:-1])
            }
            reservations.append(reservation)
        return reservations

    def book_reservation(self, sessionid, roomid, start, end):
        """ Book a reservation given the session id, the room id as an integer, and the start and end time as datetimes. """
        duration = int((end - start).seconds / 60)
        booking_url = "{}/reserve/{}/{}/?d={}".format(BASE_URL, roomid, start.strftime("%Y-%m-%dT%H:%M:%S-05:00"), duration)
        resp = requests.get(booking_url, cookies={"sessionid": sessionid})
        resp.raise_for_status()

        csrfheader = re.search(r"csrftoken=(.*?);", resp.headers["Set-Cookie"]).group(1)
        csrftoken = re.search(r"<input name=\"csrfmiddlewaretoken\" type=\"hidden\" value=\"(.*?)\"/>", resp.content.decode("utf8")).group(1)

        start_string = start.strftime("%I:%M %p")
        if start_string[0] == "0":
            start_string = start_string[1:]

        resp = requests.post(
            booking_url,
            cookies={"sessionid": sessionid, "csrftoken": csrfheader},
            headers={"Referer": booking_url},
            data={
                "csrfmiddlewaretoken": csrftoken,
                "room": roomid,
                "start_time": start_string,
                "end_time": end.strftime("%a %b %d %H:%M:%S %Y"),
                "date": start.strftime("%B %d, %Y")
            }
        )
        resp.raise_for_status()
        content = resp.content.decode("utf8")
        if "errorlist" in content:
            error_msg = re.search(r"class=\"errorlist\"><li>(.*?)</li>", content).group(1)
            return {"success": False, "error": error_msg}

        return {"success": True}

    def delete_booking(self, sessionid, booking_id):
        """ Deletes a Wharton GSR Booking for a given booking and session id. """
        url = "{}{}{}/".format(BASE_URL, "/delete/", booking_id)
        cookies = dict(sessionid=sessionid)

        try:
            resp = requests.get(url, cookies=cookies, headers={'Referer': '{}{}'.format(BASE_URL, "/reservations/")})
        except resp.exceptions.HTTPError as error:
            raise APIError("Server Error: {}".format(error))

        if resp.status_code == 404:
            raise APIError("Booking could not be found on server.")

        html = resp.content.decode("utf8")
        if "https://weblogin.pennkey.upenn.edu" in html:
            raise APIError("Wharton Auth Failed. Session ID is not valid.")

        resp.raise_for_status()

        soup = BeautifulSoup(html, "html5lib")
        middleware_token = soup.find("input", {'name': "csrfmiddlewaretoken"}).get('value')

        csrftoken = resp.cookies['csrftoken']
        cookies2 = {'sessionid': sessionid, 'csrftoken': csrftoken}
        headers = {'Referer': url}
        payload = {'csrfmiddlewaretoken': middleware_token}

        try:
            resp2 = requests.post(url, cookies=cookies2, data=payload, headers=headers)
        except resp2.exceptions.HTTPError as error:
            raise APIError("Server Error: {}".format(error))

        return {"success": True}

    def get_wharton_gsrs(self, sessionid, date):
        """ Make a request to retrieve Wharton GSR listings. """
        if date:
            date += " 05:00"
        else:
            date = datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%S")
        resp = requests.get('https://apps.wharton.upenn.edu/gsr/api/app/grid_view/', params={
            'search_time': date
        }, cookies={
            'sessionid': sessionid
        })
        if resp.status_code == 200:
            return resp.json()
        else:
            raise APIError('Remote server returned status code {}.'.format(resp.status_code))

    def switch_format(self, gsr):
        """ Convert the Wharton GSR format into the studyspaces API format. """
        if "error" in gsr:
            return gsr
        rooms = {
            "cid": 1,
            "name": "Huntsman Hall",
            "rooms": []
        }

        for time in gsr["times"]:
            for entry in time:
                entry["name"] = "GSR " + entry["room_number"]
                del entry["room_number"]
                start_time_str = entry["start_time"]
                end_time = datetime.datetime.strptime(start_time_str[:-6], '%Y-%m-%dT%H:%M:%S') + datetime.timedelta(minutes=30)
                end_time_str = end_time.strftime("%Y-%m-%dT%H:%M:%S") + "-05:00"
                time = {
                    "available": not entry["reserved"],
                    "start": entry["start_time"],
                    "end": end_time_str,
                }
                exists = False
                for room in rooms["rooms"]:
                    if room["name"] == entry["name"]:
                        room["times"].append(time)
                        exists = True
                if not exists:
                    del entry["booked_by_user"]
                    del entry["building"]
                    if "reservation_id" in entry:
                        del entry["reservation_id"]
                    entry["lid"] = 1
                    entry["capacity"] = 5
                    entry["room_id"] = entry["id"]
                    del entry["id"]
                    entry["times"] = [time]
                    del entry["reserved"]
                    del entry["end_time"]
                    del entry["start_time"]
                    rooms["rooms"].append(entry)
        return {"categories": [rooms]}

    def get_wharton_gsrs_formatted(self, sessionid):
        """ Return the wharton GSR listing formatted in studyspaces format. """
        gsrs = self.get_wharton_gsrs(sessionid, None)
        return self.switch_format(gsrs)



__version__ = '1.8'

from .registrar import Registrar
from .directory import Directory
from .dining import Dining
from .dining import DiningV2
from .transit import Transit
from .news import News
from .laundry import Laundry
from .map import Map
from .studyspaces import StudySpaces
from .calendar3year import Calendar
from .fitness import Fitness
from .wharton import Wharton



import re
from requests import get


class APIError(ValueError):
    pass


class WrapperBase(object):

    def __init__(self, bearer, token):
        self.bearer = bearer
        self.token = token

    @property
    def headers(self):
        """The HTTP headers needed for signed requests"""
        return {
            "Authorization-Bearer": self.bearer,
            "Authorization-Token": self.token,
        }

    def _request(self, url, params=None):
        """Make a signed request to the API, raise any API errors, and
        returning a tuple of (data, metadata)
        """

        response = get(url, params=params, headers=self.headers, timeout=30)

        if response.status_code != 200:
            raise APIError('Request to {} returned {}'.format(response.url, response.status_code))

        response = response.json()

        error_text = response['service_meta']['error_text']
        if error_text:
            raise APIError(error_text)

        return response

    @staticmethod
    def validate(validation_dict, params):
        errors = {}
        params_map = validation_dict['acceptable_search_url_parameters_map']
        for param in params:
            if param not in params_map:
                errors[param] = 'This is not a valid parameter'
            else:
                match = re.match(r"Use one of the values from the map (\w+)",
                                 params_map[param], flags=re.IGNORECASE)
                if match:
                    map_name = match.group(1)
                    d = validation_dict[map_name]
                    if params[param] not in d and params[param].upper() not in d:
                        errors[param] = 'Invalid value for this parameter'
        return errors



from os import path
from .base import WrapperBase


BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/"
ENDPOINTS = {
    'CATALOG': BASE_URL + 'course_info',
    'SEARCH': BASE_URL + 'course_section_search',
    'SEARCH_PARAMS': BASE_URL + 'course_section_search_parameters'
}


class Registrar(WrapperBase):
    """The client for the Registrar. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import Registrar
      >>> r = Registrar('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """
    def __init__(self, bearer, token):
        WrapperBase.__init__(self, bearer, token)
        self.val_info = None

    def _iter_response(self, url, params=None):
        """Return an enumerable that iterates through a multi-page API request"""
        if params is None:
            params = {}
        params['page_number'] = 1

        # Last page lists itself as next page
        while True:
            response = self._request(url, params)

            for item in response['result_data']:
                yield item

            # Last page lists itself as next page
            if response['service_meta']['next_page_number'] == params['page_number']:
                break

            params['page_number'] += 1

    def search(self, params, validate=False):
        """Return a generator of section objects for the given search params.

        :param params: Dictionary of course search parameters.
        :param validate: Optional. Set to true to enable request validation.

        >>> cis100s = r.search({'course_id': 'cis', 'course_level_at_or_below': '200'})
        """
        if self.val_info is None:
            self.val_info = self.search_params()

        if validate:
            errors = self.validate(self.val_info, params)
        if not validate or len(errors) == 0:
            return self._iter_response(ENDPOINTS['SEARCH'], params)
        else:
            return {'Errors': errors}

    def course(self, dept, course_number):
        """Return an object of semester-independent course info. All arguments
        should be strings.

        >>> cis120 = r.course('cis', '120')
        """
        response = self._request(path.join(ENDPOINTS['CATALOG'], dept, course_number))
        return response['result_data'][0]

    def department(self, dept):
        """Return an iterator of all course-info objects in a department, in no
        particular order.
        """
        return self._iter_response(path.join(ENDPOINTS['CATALOG'], dept))

    def section(self, dept, course_number, sect_number):
        """Return a single section object for the given section. All arguments
        should be strings. Throws a `ValueError` if the section is not found.

        >>> lgst101_bfs = r.course('lgst', '101', '301')
        """
        section_id = dept + course_number + sect_number
        sections = self.search({'course_id': section_id})
        try:
            return next(sections)
        except StopIteration:
            raise ValueError('Section %s not found' % section_id)

    def search_params(self):
        """Return a dictionary of possible search parameters and their possible
        values and descriptions.
        """
        return self._request(ENDPOINTS['SEARCH_PARAMS'])['result_data'][0]



"""A module for consuming the Penn Libraries API"""
import requests

BASE_URL = "http://dla.library.upenn.edu/2.0.0/search"


def search(query):
    """Search Penn Libraries Franklin for documents
    The maximum pagesize currently is 50.
    """
    params = {
        's.cmd': 'setTextQuery(%s)setPageSize(50)setHoldingsOnly(true)' % query
    }
    return requests.get(BASE_URL, params=params, timeout=10).json()



from setuptools import setup


setup(
    name='PennSDK',
    description='Python tools for building Penn-related applications',
    url='https://github.com/pennlabs/penn-sdk-python',
    author='Penn Labs',
    author_email='admin@pennlabs.org',
    version='1.8.1',
    packages=['penn'],
    license='MIT',
    package_data={
        # If any package contains *.txt or *.rst files, include them:
        '': ['*.txt', '*.rst'],
        'penn': ['data/laundry.csv'],
    },
    long_description=open('./README.rst').read(),
    install_requires=[
        'nameparser==0.5.6',
        'requests==2.20.0',
        'beautifulsoup4==4.6.0',
        'html5lib==1.0.1'
    ]
)



import requests
import datetime


BASE_URL = "https://www.google.com/calendar/ical/pennalmanac@gmail.com/public/basic.ics"


class Calendar(object):
    def __init__(self):
        pass

    def pull_3year(self):
        """Returns a list (in JSON format) containing all the events from the Penn iCal Calendar.

        List contains events in chronological order.

        Each element of the list is a dictionary, containing:
            - Name of the event 'name'
            - Start date 'start'
            - End date 'end'
        """
        events = []
        r = requests.get(BASE_URL).text
        lines = r.split("\r\n")
        d = {}
        for line in lines:
            if line == "BEGIN:VEVENT":
                d = {}
            elif line[:7] == "DTSTART":
                raw_date = line.split(":")[1]
                start_date = datetime.datetime.strptime(raw_date, '%Y%m%d').date()
                d['start'] = start_date.strftime('%Y-%m-%d')
            elif line[:5] == "DTEND":
                raw_date = line.split(":")[1]
                end_date = datetime.datetime.strptime(raw_date, '%Y%m%d').date()
                d['end'] = end_date.strftime('%Y-%m-%d')
            elif line[:7] == "SUMMARY":
                name = line.split(":")[1]
                d['name'] = str(name).strip()
            elif line == "END:VEVENT":
                events.append(d)

        events.sort(key=lambda d: d['start'])
        return events



"""A module for consuming the Penn Dining API"""
import datetime
from .base import WrapperBase


BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/dining/"
V2_BASE_URL = "https://esb.isc-seo.upenn.edu/8091/open_data/dining/v2/?service="

ENDPOINTS = {
    'MENUS': BASE_URL + 'menus',
    'VENUES': BASE_URL + 'venues',
}

V2_ENDPOINTS = {
    'VENUES': V2_BASE_URL + 'venues',
    'HOURS': V2_BASE_URL + 'cafes&cafe=',
    'MENUS': V2_BASE_URL + 'menus&cafe=',
    'ITEMS': V2_BASE_URL + 'items&item='
}

VENUE_NAMES = {
    '593': '1920 Commons',
    '636': 'Hill House',
    '637': 'Kings Court English House',
    '638': 'Kosher Dining at Falk'
}


def normalize_weekly(data):
    """Normalization for dining menu data"""
    if "tblMenu" not in data["result_data"]["Document"]:
        data["result_data"]["Document"]["tblMenu"] = []
    if isinstance(data["result_data"]["Document"]["tblMenu"], dict):
        data["result_data"]["Document"]["tblMenu"] = [data["result_data"]["Document"]["tblMenu"]]
    for day in data["result_data"]["Document"]["tblMenu"]:
        if "tblDayPart" not in day:
            continue
        if isinstance(day["tblDayPart"], dict):
            day["tblDayPart"] = [day["tblDayPart"]]
        for meal in day["tblDayPart"]:
            if isinstance(meal["tblStation"], dict):
                meal["tblStation"] = [meal["tblStation"]]
            for station in meal["tblStation"]:
                if isinstance(station["tblItem"], dict):
                    station["tblItem"] = [station["tblItem"]]
    return data


def get_meals(v2_response, building_id):
    """Extract meals into old format from a DiningV2 JSON response"""
    result_data = v2_response["result_data"]
    meals = []
    day_parts = result_data["days"][0]["cafes"][building_id]["dayparts"][0]
    for meal in day_parts:
        stations = []
        for station in meal["stations"]:
            items = []
            for item_id in station["items"]:
                item = result_data["items"][item_id]
                new_item = {}
                new_item["txtTitle"] = item["label"]
                new_item["txtPrice"] = ""
                new_item["txtNutritionInfo"] = ""
                new_item["txtDescription"] = item["description"]
                new_item["tblSide"] = ""
                new_item["tblFarmToFork"] = ""
                attrs = [{"description": item["cor_icon"][attr]} for attr in item["cor_icon"]]
                if len(attrs) == 1:
                    new_item["tblAttributes"] = {"txtAttribute": attrs[0]}
                elif len(attrs) > 1:
                    new_item["tblAttributes"] = {"txtAttribute": attrs}
                else:
                    new_item["tblAttributes"] = ""
                if isinstance(item["options"], list):
                    item["options"] = {}
                if "values" in item["options"]:
                    for side in item["options"]["values"]:
                        new_item["tblSide"] = {"txtSideName": side["label"]}
                items.append(new_item)
            stations.append({"tblItem": items, "txtStationDescription": station["label"]})
        meals.append({"tblStation": stations, "txtDayPartDescription": meal["label"]})
    return meals


class DiningV2(WrapperBase):
    """The client for the Registrar. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import DiningV2
      >>> din = DiningV2('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """
    def venues(self):
        """Get a list of all venue objects.

          >>> venues = din.venues()
        """
        response = self._request(V2_ENDPOINTS['VENUES'])
        return response

    def hours(self, venue_id):
        """Get the list of hours for the venue corresponding to
        venue_id.

        :param venue_id:
          A string representing the id of a venue, e.g. "abc".


        >>> commons_hours = din.hours("593")
        """
        response = self._request(V2_ENDPOINTS['HOURS'] + venue_id)
        return response

    def menu(self, venue_id, date):
        """Get the menu for the venue corresponding to venue_id,
        on date.

        :param venue_id:
          A string representing the id of a venue, e.g. "abc".
        :param date:
          A string representing the date of a venue's menu, e.g. "2015-09-20".


        >>> commons_menu = din.menu("593", "2015-09-20")
        """
        query = "&date=" + date
        response = self._request(V2_ENDPOINTS['MENUS'] + venue_id + query)
        return response

    def item(self, item_id):
        """Get a description of the food item corresponding to item_id.

        :param item_id:
          A string representing the id of an item, e.g. "3899220".


        >>> tomato_sauce = din.item("3899220")
        """
        response = self._request(V2_ENDPOINTS['ITEMS'] + item_id)
        return response


class Dining(WrapperBase):
    """The client for the Registrar. Used to make requests to the API.

    :param bearer: The user code for the API
    :param token: The password code for the API

    Usage::

      >>> from penn import Dining
      >>> din = Dining('MY_USERNAME_TOKEN', 'MY_PASSWORD_TOKEN')
    """
    def venues(self):
        """Get a list of all venue objects.

          >>> venues = din.venues()
        """
        response = self._request(V2_ENDPOINTS['VENUES'])
        # Normalize `dateHours` to array
        for venue in response["result_data"]["document"]["venue"]:
            if venue.get("id") in VENUE_NAMES:
                venue["name"] = VENUE_NAMES[venue.get("id")]
            if isinstance(venue.get("dateHours"), dict):
                venue["dateHours"] = [venue["dateHours"]]
            if "dateHours" in venue:
                for dh in venue["dateHours"]:
                    if isinstance(dh.get("meal"), dict):
                        dh["meal"] = [dh["meal"]]
        return response

    def menu_daily(self, building_id):
        """Get a menu object corresponding to the daily menu for the
        venue with building_id.

        :param building_id:
          A string representing the id of a building, e.g. "abc".


        >>> commons_today = din.menu_daily("593")
        """
        today = str(datetime.date.today())
        v2_response = DiningV2(self.bearer, self.token).menu(building_id, today)
        response = {'result_data': {'Document': {}}}
        response["result_data"]["Document"]["menudate"] = datetime.datetime.strptime(today, '%Y-%m-%d').strftime('%-m/%d/%Y')
        if building_id in VENUE_NAMES:
            response["result_data"]["Document"]["location"] = VENUE_NAMES[building_id]
        else:
            response["result_data"]["Document"]["location"] = v2_response["result_data"]["days"][0]["cafes"][building_id]["name"]
        response["result_data"]["Document"]["tblMenu"] = {"tblDayPart": get_meals(v2_response, building_id)}
        return response

    def menu_weekly(self, building_id):
        """Get an array of menu objects corresponding to the weekly menu for the
        venue with building_id.

        :param building_id:
            A string representing the id of a building, e.g. "abc".

        >>> commons_week = din.menu_weekly("593")
        """
        din = DiningV2(self.bearer, self.token)
        response = {'result_data': {'Document': {}}}
        days = []
        for i in range(7):
            date = str(datetime.date.today() + datetime.timedelta(days=i))
            v2_response = din.menu(building_id, date)
            if building_id in VENUE_NAMES:
                response["result_data"]["Document"]["location"] = VENUE_NAMES[building_id]
            else:
                response["result_data"]["Document"]["location"] = v2_response["result_data"]["days"][0]["cafes"][building_id]["name"]
            formatted_date = datetime.datetime.strptime(date, '%Y-%m-%d').strftime('%-m/%d/%Y')
            days.append({"tblDayPart": get_meals(v2_response, building_id), "menudate": formatted_date})
        response["result_data"]["Document"]["tblMenu"] = days
        return normalize_weekly(response)



import re
import requests
import datetime
import pytz
from bs4 import BeautifulSoup

FITNESS_URL = "https://connect2concepts.com/connect2/?type=bar&key=650471C6-D72E-4A16-B664-5B9C3F62EEAC"
CALENDAR_URL = "https://api.teamup.com/ks13d3ccc86a21d29e/events"


class Fitness(object):
    """Used to interact with the Penn Recreation usage pages.

    Usage::

        >>> from penn import Fitness
        >>> fit = Fitness('SCHEDULE_TOKEN')
        >>> fit.get_usage()
    """

    def __init__(self, schedule_token):
        self.token = schedule_token

    def get_schedule(self):
        resp = requests.get(CALENDAR_URL, timeout=30, headers={
            "Teamup-Token": self.token
        })
        resp.raise_for_status()
        raw_data = resp.json()
        data = {}
        for item in raw_data["events"]:
            name = re.sub(r"\s*(Hours)?\s*-?\s*(CLOSED|OPEN)?$", "", item["title"], re.I).rsplit("-", 1)[0].strip().title()
            out = {
                "all_day": item["all_day"]
            }
            if not item["all_day"]:
                out["start"] = item["start_dt"]
                out["end"] = item["end_dt"]
            else:
                out["day"] = item["start_dt"].split("T")[0]
            if name not in data:
                data[name] = {
                    "name": name,
                    "hours": []
                }
            data[name]["hours"].append(out)
        return list(data.values())

    def get_usage(self):
        """Get fitness locations and their current usage."""

        resp = requests.get(FITNESS_URL, timeout=30)
        resp.raise_for_status()

        soup = BeautifulSoup(resp.text, "html5lib")
        eastern = pytz.timezone('US/Eastern')
        output = []
        for item in soup.findAll("div", {"class": "barChart"}):
            data = [x.strip() for x in item.get_text("\n").strip().split("\n")]
            data = [x for x in data if x]
            name = re.sub(r"\s*(Hours)?\s*-?\s*(CLOSED|OPEN)?$", "", data[0], re.I).strip()
            output.append({
                "name": name,
                "open": "Open" in data[1],
                "count": int(data[2].rsplit(" ", 1)[-1]),
                "updated": eastern.localize(datetime.datetime.strptime(data[3][8:].strip(), '%m/%d/%Y %I:%M %p')).isoformat(),
                "percent": int(data[4][:-1])
            })
        return output
